/* Argument checker for killall */
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "signame.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#if defined(HAVE_LINUX) || defined(HAVE_IRIX)
char *progname = "killall";

/*D

  ptkillall - SUT killall

Description:

ptkillall kills named processes on given hosts

Command Line Arguments:

ptkillall [-m <machine_file>|-M <machine_list>] [<signal>] <pname>

.N machine_args
. <signal> - The signal to send.  Can be a number or a name.  If not given,
the default signal is sent (see NOTES below and killall(1), pkill(1)).
- <pname> - The name of the process to kill

Environment Variables:

.N machine_envs

Examples:

To kill all processes named 'runaway' on the machines specified in the 
file "destination_machines" use

.vb
ptkillall -m destination_machines runaway
.ve

To kill the same processes, but specifying the machines on the command line, use

.vb
ptkillall -M "host1 host2 host3" runaway
.ve

To kill the same processes, but specifying the signal 9, use

.vb
ptkillall -m destination_machines -9 runaway
.ve

To kill the same processes, but specifying the signal SIGKILL, use

.vb
ptkillall -m destination_machines SIGKILL runaway
.ve

Notes:

This command simply calls killall(1) on Linux and IRIX.  It calls pkill(1) on
Solaris.  It is unavailable on other systems.

.seealso killall(1), pkill(1)
  
 D*/

#elif defined(HAVE_SOLARIS)
char *progname = "pkill";

/*D

  ptpkill - SUT killall

Description:

ptpkill kills named processes on given hosts

Command Line Arguments:

ptpkill [-m <machine_file>|-M <machine_list>] [signal] <pname>

.N machine_args
. [signal] - The signal to send.  Can be a number or a name.  If not given,
the default signal is sent (see NOTES below and killall(1), pkill(1)).
- <pname> - The name of the process to kill

Environment Variables:

.N machine_envs

Examples:

To kill all processes named 'runaway' on the machines specified in the 
file "destination_machines" use

.vb
ptpkill -m destination_machines runaway
.ve

To kill the same processes, but specifying the machines on the command line, use

.vb
ptpkill -M "host1 host2 host3" runaway
.ve

To kill the same processes, but specifying the signal 9, use

.vb
ptpkill -m destination_machines -9 runaway
.ve

To kill the same processes, but specifying the signal SIGKILL, use

.vb
ptpkill -m destination_machines SIGKILL runaway
.ve

Notes:

This command simply calls killall(1) on Linux and IRIX.  It calls pkill(1) on
Solaris.  It is unavailable on other systems.

.seealso killall(1), pkill(1)
  
 D*/

#endif /* defined(HAVE_LINUX) || defined(HAVE_IRIX) */
 
int check_args(int argc, char *argv[])
{
    int dummy;

    switch (argc) {
        case 2:
            return 0;
        case 3:
            if (argv[1][0] == '-')
                argv[1]++;

            if (sscanf(argv[1], "%d", &dummy) == 1)
                return 0;

            if (signame_to_signum(argv[1]) >= 0)
                return 0;
        default:
            return -1;
    }
}
