#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <signal.h>
#include <string.h>
#include <sys/wait.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>
#include "get_date.h"
#include "ptfps.h"
#include "ptfps_aix.h"
#include "debug.h"

extern int getprocs (struct procsinfo *,int,struct fdsinfo *,int,pid_t *,int);

process_t *get_next_process()
{
    static pid_t ndx = 0;
    static process_t pr;

    if (getprocs(&pr, sizeof(pr), NULL, 0, &ndx, 1) != 1) {
        ndx = 0;
        return NULL;
    }

    return &pr;
}

/* UID's */
uid_t get_ruid(process_t *p)
{
    return p->pi_cred.cr_ruid;
}
uid_t get_euid(process_t *p)
{
    return p->pi_cred.cr_uid;
}
uid_t get_suid(process_t *p)
{
    return p->pi_cred.cr_suid;
}

/* GID's */
gid_t get_rgid(process_t *p)
{
    return p->pi_cred.cr_rgid;
}

gid_t get_egid(process_t *p)
{
    return p->pi_cred.cr_gid;
}

gid_t get_sgid(process_t *p)
{
    return p->pi_cred.cr_sgid;
}

/* Username's */
char *get_rusername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_ruid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

char *get_eusername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_euid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

char *get_susername(process_t *p)
{
    struct passwd *pwd;

    pwd = getpwuid(get_suid(p));
    if (pwd == NULL)
        return "N/A";

    return pwd->pw_name;
}

/* Group names */
char *get_rgroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_rgid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

char *get_egroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_egid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

char *get_sgroupname(process_t *p)
{
    struct group *gr;

    gr = getgrgid(get_sgid(p));
    if (gr == NULL)
        return "N/A";

    return gr->gr_name;
}

/* Etc */
char *get_name(process_t *p)
{
    static char buf[256];
    char *tmp;

    if (getargs(p, sizeof(process_t), buf, sizeof(buf)) != 0)
        return p->pi_comm;

    tmp = buf;
    while (1) {
        int len = strlen(tmp);
        if (len == 0)
            break;
        tmp += len;
        if (tmp - buf > sizeof(buf))
            break;
        *tmp = ' ';
    }

    return buf;
}

char *get_cmd(process_t *p)
{
    return p->pi_comm;
}

static char *brute_tty(dev_t dev)
{
    DIR *d;
    struct dirent *de;
    struct stat st;
    static char buf[50];

    d = opendir("/dev/pts");
    if (d == NULL)
        return "?";

    while ((de = readdir(d)) != NULL) {
        buf[0] = '\0';
        strcat(buf, "/dev/pts/");
        strncat(buf, de->d_name, 50-sizeof("/dev/pts/"));
        if (stat(buf, &st) == 0) {
            if (st.st_rdev == dev) {
                strcpy(buf, "pts/");
                strncat(buf, de->d_name, 50-sizeof("pts/"));
                return buf;
            }
        }
    }

    d = opendir("/dev");
    if (d == NULL)
        return "?";

    while ((de = readdir(d)) != NULL) {
        strcpy(buf, "/dev/");
        strncat(buf, de->d_name, 50-sizeof("/dev/"));
        if (stat(buf, &st) == 0) {
            if (st.st_rdev == dev) {
                strncpy(buf, de->d_name, 50);
                return buf;
            }
        }
    }

    return "?";
}

char *get_tty(process_t *p)
{
    if (p->pi_ttyp)
        return brute_tty((dev_t)(p->pi_ttyd));

    return "?";
}

pid_t get_pid(process_t *p)
{
    return p->pi_pid;
}

pid_t get_ppid(process_t *p)
{
    return p->pi_ppid;
}

time_t get_stime(process_t *p)
{
    return (time_t)(p->pi_start);
}

unsigned long get_time(process_t *p)
{
    return p->pi_utime + p->pi_stime;
}

double get_pcpu(process_t *p)
{
    return get_time(p)/(time(NULL) - get_stime(p));
}

double get_pmem(process_t *p)
{
    return p->pi_prm;
}

unsigned long get_rss(process_t *p)
{
    return p->pi_drss + p->pi_trss;
}

unsigned long get_vsz(process_t *p)
{
    return p->pi_dvm;
}

unsigned long get_flags(process_t *p)
{
    return p->pi_flags;
}

int get_pri(process_t *p)
{
    return getpri(get_pid(p));
}

char *get_wchan(process_t *p)
{
    return "";
}

int get_nice(process_t *p)
{
    return p->pi_nice;
}

status_t get_state(process_t *p)
{
    switch (p->pi_state) {
        case SACTIVE:
            return ST_RUNNABLE;
        case SSWAP:
            return ST_SLEEPING;
        case SSTOP:
            return ST_STOPPED;
        case SZOMB:
            return ST_ZOMBIE;
        default:
            return ST_UNKNOWN;
    }
}
