#ifndef PTOOLS_H
#define PTOOLS_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#ifndef HAVE_STRDUP
extern char *strdup(const char *str);
#endif /* HAVE_STRDUP */

#ifndef HAVE_SNPRINTF
extern int snprintf (char *str, size_t count, const char *fmt, ...);
#endif /* HAVE_SNPRINTF */

#ifdef MIN
#undef MIN
#endif /* MIN */
#define MIN(x, y) ((x) < (y) ? (x) : (y))

#ifdef MAX
#undef MAX
#endif /* MAX */
#define MAX(x, y) ((x) > (y) ? (x) : (y))

#define WHITESPACE      " \n\t\v\f\r"

/*@
  SUT - Scalable Unix Tools

Description:
SUT is a collection of commands that aim to create a Unix environment on
parallel machines (e.g. Beowulf clusters, etc.) that is analogous to the
uniprocessor Unix environment.  SUT consists of the following commands\:

+ ptcat - Parallel cat
. ptchgrp - Parallel chgrp
. ptchmod - Parallel chmod
. ptchown - Parallel chown
. ptcp - Parallel cp
. ptdistrib - Parallel distribute
. ptexec - Parallel command execution
. ptfind - Parallel find
. ptfps - Parallel find in process space
. ptkillall - Parallel killall (Linux semantics)
. ptln - Parallel ln
. ptls - Parallel ls
. ptmkdir - Parallel mkdir
. ptmv - Parallel mv
. ptpkill - Parallel pkill (Solaris semantics)
. ptpred - Parallel predicate
. ptrm - Parallel rm
. ptrmdir - Parallel rmdir
- pttest[ao] - Parallel test

All of these commands share a common command line syntax\:

pt<command name> [-all|-m <machine_file>|-M <machine_list>] <arguments>

The <command name> and <arguments> are specific to each command, but the
first argument or first two arguments are always of the same form.  Because
these tools are intended for a parallel environment, it is necessary to
specify machines on which to run them.  This first argument[s] is that
specifier.  The first way to specify machines is explicitly.  The -m and -M
options allow this.  They both take

Node Syntax:

Nodes can be specified     

.vb
    Nodelist -> <empty>
    Nodelist -> Nodename Nodelist
    Nodelist -> Nodeset Nodelist
    
    Nodename -> <string>
    
    Nodeset -> Format `@' Rangelist
    
    Format -> <string> `%d' <string>
    
    Rangelist -> Range
    Rangelist -> Range `,' Rangelist
    
    Range -> <integer>
    Range -> <integer> `-' <integer>
.ve

+ enumnodes - Node enumerator
- ptdisp - Parallel data display tool
@*/

#endif /* PTOOLS_H */
